@@ -3,18 +3,10 @@ package rhcos
 import (
 	"context"
 	"encoding/json"
-	"fmt"
 	"io/ioutil"
-	"os"
 
 	"github.com/openshift/installer/data"
 	"github.com/pkg/errors"
-
-	"github.com/openshift/installer/pkg/types"
-)
-
-var (
-	errInvalidArch = fmt.Errorf("no build metadata for given architecture")
 )
 
 type metadata struct {
@@ -32,28 +24,28 @@ type metadata struct {
 	BaseURI string `json:"baseURI"`
 	Images  struct {
 		QEMU struct {
-			Path   string `json:"path"`
-			SHA256 string `json:"sha256"`
+			Path               string `json:"path"`
+			SHA256             string `json:"sha256"`
+			UncompressedSHA256 string `json:"uncompressed-sha256"`
 		} `json:"qemu"`
 		OpenStack struct {
-			Path   string `json:"path"`
-			SHA256 string `json:"sha256"`
+			Path               string `json:"path"`
+			SHA256             string `json:"sha256"`
+			UncompressedSHA256 string `json:"uncompressed-sha256"`
 		} `json:"openstack"`
 	} `json:"images"`
 	OSTreeVersion string `json:"ostree-version"`
 }
 
-func fetchRHCOSBuild(ctx context.Context, arch types.Architecture) (*metadata, error) {
-	file, err := data.Assets.Open(fmt.Sprintf("rhcos-%s.json", arch))
+func fetchRHCOSBuild(ctx context.Context) (*metadata, error) {
+	file, err := data.Assets.Open("rhcos.json")
 	if err != nil {
 		return nil, err
 	}
 	defer file.Close()
 
 	body, err := ioutil.ReadAll(file)
-	if os.IsNotExist(err) {
-		return nil, errInvalidArch
-	} else if err != nil {
+	if err != nil {
 		return nil, err
 	}
 
